#[derive(Debug, Clone)]
pub enum IdentifierType {
    User,
}

impl serde::Serialize for IdentifierType {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: serde::Serializer,
    {
        serializer.serialize_str(match self {
            IdentifierType::User => "m.id.user",
        })
    }
}

impl<'de> serde::Deserialize<'de> for IdentifierType {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: serde::Deserializer<'de>,
    {
        struct IdentifierVisitor;

        impl<'de> serde::de::Visitor<'de> for IdentifierVisitor {
            type Value = IdentifierType;

            fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
                formatter.write_str("Identifier")
            }

            fn visit_borrowed_str<E>(self, v: &'de str) -> Result<Self::Value, E>
            where
                E: serde::de::Error,
            {
                match v {
                    "m.id.user" => Ok(IdentifierType::User),
                    _ => Err(serde::de::Error::custom("Unknown identifier")),
                }
            }
        }

        deserializer.deserialize_str(IdentifierVisitor {})
    }
}
